@@ -9,9 +9,9 @@ class NextstrainCliError(Exception):
     pass
 
 class UserError(NextstrainCliError):
-    def __init__(self, message):
+    def __init__(self, message, *args, **kwargs):
         # Remove leading newlines, trailing whitespace, and then indentation
         # to better support nicely-formatted """multi-line strings""".
-        formatted_message = dedent(message.lstrip("\n").rstrip())
+        formatted_message = dedent(message.lstrip("\n").rstrip()).format(*args, **kwargs)
 
         super().__init__("Error: " + formatted_message)
